EnableExplicit

#LibBuild_DebugMemory = #False ; PureBasic's profiler is superior
#LibBuild_EstMemOverHeadBytesPerAlloc = 32

CompilerIf Not Defined(IncludePath, #PB_Constant)
  ;#IncludePath = #PB_Compiler_Home + "\ctlib\"
  #IncludePathInc = #PB_Compiler_FilePath + "inc\"
  CompilerEndIf

  Macro LibBuild_MacroDoubleQuote
      "
    EndMacro

;LibBuild_Add    Global   loads/adds modules, procedures, macros etc
;LibBuild_Init   InProc   initializes / run procedures
;LibBuild_Close  InProc   Cleanup / run cleanup procedures
;LibBuild_Done   LastLn   Adds Cleanup procedure(s)

; Global = no in procedure!
; InProc = Anywhere, but in procedure if DLL
; LastLN = Your last line of code

Declare LibBuild_InitRealPost(AppName.s)
Declare LibBuild_Close()
CompilerIf #PB_Compiler_Debugger
;  Declare.i WinSock_GetHostByName(xHostname.s)
  CompilerEndIf

Procedure LibBuild_InitRealPre()
  Global hInstance.i
  CompilerIf #PB_Compiler_Debugger And #LibBuild_DebugMemory
    Global MemCounts.i
    CompilerEndIf
  CompilerIf Not #PB_Compiler_Processor = #PB_Processor_x64
    Global IsHighColor.i
    CompilerIf #PB_Compiler_Unicode
      Global IsMinWindows2000.i
      Global IsMinWindowsXP.i
      CompilerElse
      Global IsMinWindows98.i
      Global IsMinWindowsME.i
      CompilerEndIf
    CompilerEndIf
  Global IsMinWindowsVista.i
  Global IsMinWindows7.i
  Global *ptrTemp
  ;Global HtmlHelp_LastCall.i
  ;Global HtmlHelp_Active.i
  #DoeventsEx_SleepValueLow = 25
  #DoeventsEx_SleepValueHigh = 100
  #AsyncKeyPressedMask = $8000

  #ptrTempSize = 32000
  CompilerIf #PB_Compiler_Unicode
    #ptrTempSizeXAW = (#ptrTempSize >> 1) - 1
    CompilerElse
    #ptrTempSizeXAW = #ptrTempSize - 1
    CompilerEndIf
  EndProcedure

CompilerIf #PB_Compiler_Unicode
  Procedure.i GetProcAddress(*xModule, xName.s)
    PokeS(*ptrTemp, xName, -1, #PB_Ascii)
    ProcedureReturn GetProcAddress_(*xModule, *ptrTemp)
    EndProcedure
  Macro GetProcAddressAW(xModule, FncNoAW)
    GetProcAddress(xModule, FncNoAW + "W")
    EndMacro
  CompilerElse
  Macro GetProcAddress(xModule, xName)
    GetProcAddress_(xModule, xName)
    EndMacro
  Macro GetProcAddressAW(xModule, FncNoAW)
    GetProcAddress_(xModule, FncNoAW + "A")
    EndMacro
  CompilerEndIf

CompilerIf #PB_Compiler_Debugger And #LibBuild_DebugMemory
  Procedure.i MemAlloc(xSize.i)
    Define.i xResult
    
    xResult = AllocateMemory(xSize + 4)
    If xResult
      ;Debug "Alloc: " + Hex(xResult) + " / " + Str(xResult)
      PokeL(xResult, $15aa110c)
      MemCounts + 1
      xResult + 4
      EndIf
    ProcedureReturn xResult
    EndProcedure
  Procedure.i MemAllocNC(xSize.i)
    Define.i xResult

    xResult = AllocateMemory(xSize + 4, #PB_Memory_NoClear)
    If xResult
      ;Debug "Alloc: " + Hex(xResult) + " / " + Str(xResult)
      PokeL(xResult, $15aa110c)
      MemCounts + 1
      xResult + 4
      EndIf
    ProcedureReturn xResult
    EndProcedure
  Procedure MemCopyRTL(*Destination, *Source, NumBytes)
    CopyMemory(*Source, *Destination, NumBytes)
    EndProcedure
  Procedure MemCopyPB(Source, Destination, NumBytes)
    MemCopyRTL(Destination, Source, NumBytes)
    EndProcedure
  Procedure MemFree(*ptrMem)
    If *ptrMem
      ;$01da110c
      If PeekL(*ptrMem - 4) = $15aa110c
        FreeMemory(*ptrMem-4)
        MemCounts - 1
        Else
        ShowCallstack()
        CallDebugger
        EndIf
      Else
      ShowCallstack()
      CallDebugger
      EndIf
    EndProcedure
  CompilerElse
  Macro MemAlloc(xSize)
    AllocateMemory(xSize)
    EndMacro
  Macro MemAllocNC(xSize)
    AllocateMemory(xSize, #PB_Memory_NoClear)
    EndMacro
  Macro MemCopyRTL(Destination, Source, NumBytes)
    CopyMemory(Source, Destination, NumBytes)
    EndMacro
  Macro MemCopyPB(Source, Destination, NumBytes)
    CopyMemory(Source, Destination, NumBytes)
    EndMacro
  Macro MemFree(ptrMem)
    FreeMemory(ptrMem)
    EndMacro
  CompilerEndIf

Macro StrS(xVal)
  Str(xVal)
  EndMacro

CompilerIf #PB_Compiler_Debugger
  Macro LibBuild_CheckPriority(ModuleLowTrg)
  ;ModuleHiChk
    CompilerIf Defined(LibBuild_#ModuleLowTrg#_Included, #PB_Constant) And Not Defined(LibBuild_PriorityError, #PB_Constant)
      ;CompilerError LibBuild_MacroDoubleQuote#ModuleHiChk" + " 1 must be included before " + LibBuild_MacroDoubleQuote#ModuleLowTrg"
      CompilerIf #LibBuild_#ModuleLowTrg#_Invoker = LibBuild_MacroDoubleQuote#ModuleLowTrg"
        #LibBuild_PriorityError = #LibBuild_#ModuleLowTrg#_Invoker
        CompilerElse
        #LibBuild_PriorityError = #LibBuild_#ModuleLowTrg#_Invoker + "(" + LibBuild_MacroDoubleQuote#ModuleLowTrg" + " relies on this)"
        CompilerEndIf
      CompilerEndIf
    EndMacro
  CompilerElse
  Macro LibBuild_CheckPriority(ModuleLowTrg)
    EndMacro
  CompilerEndIf

Macro LibBuild_Add(ModuleName)
  CompilerIf Not Defined(LibBuild_#ModuleName#_Included, #PB_Constant)
    #LibBuild_#ModuleName#_Included = #True
    #LibBuild_#ModuleName#_Invoker = LibBuild_MacroDoubleQuote#ModuleName"
    CompilerEndIf
  XIncludeFile #IncludePathInc + LibBuild_MacroDoubleQuote#ModuleName.pb"

  CompilerIf Defined(LibBuild_PriorityError, #PB_Constant)
    CompilerError LibBuild_MacroDoubleQuote#ModuleName" + " must be included before " + #LibBuild_PriorityError
    CompilerEndIf
  ;UndefineMacro LibBuild_LastAddModule
  EndMacro

Macro LibBuild_Include(ModuleName)
  CompilerIf Not Defined(LibBuild_#ModuleName#_Included, #PB_Constant)
    #LibBuild_#ModuleName#_Included = #True
    #LibBuild_#ModuleName#_Invoker = LibBuild_MacroDoubleQuote#ModuleName"
    CompilerEndIf
  XIncludeFile #IncludePathInc + LibBuild_MacroDoubleQuote#ModuleName.pb"
  CompilerIf Defined(LibBuild_PriorityError, #PB_Constant)
    CompilerError LibBuild_MacroDoubleQuote#ModuleName" + " must be included before " + #LibBuild_PriorityError
    CompilerEndIf
  EndMacro

Macro LibBuild_CondInitModule(ModuleName)
  ;Define.s xChar1 = LibBuild_MacroDoubleQuote#ModuleName#_LibBuildPostInit"
  CompilerIf Defined(LibBuild_#ModuleName#_Included, #PB_Constant)
    ModuleName#_LibBuildPostInit()
    ;xChar1 + ": Yes"
    ;CompilerElse
    ;xChar1 + ": No"
    CompilerEndIf
  ;MessageBox_(0, @xChar1, @"Test", 0)
  EndMacro

LibBuild_Include(OSMacros)
LibBuild_Include(OtherMacros)

;LibBuild_Include(Application.pb"

Procedure LibBuild_Init(AppName.s)
  LibBuild_InitRealPre()
  ProcedureReturn LibBuild_InitRealPost(AppName)
  EndProcedure

Macro LibBuild_Done()
  Procedure.i LibBuild_InitRealPost(AppName.s)
    Define.i xResult
    Define.i Major
    Define.i Minor
    Define.i xLong1
    Define.s xChar1

    CompilerIf #PB_Compiler_Debugger
      ;Define.s xChar1 ; priority warnings!
      CompilerEndIf

    If Not *ptrTemp
      *ptrTemp = MemAllocNC(#ptrTempSize)
      EndIf
    CompilerIf #PB_Compiler_Debugger
      ;LibBuild_CheckPriorityWarning...
      CompilerEndIf

    If *ptrTemp
      hInstance = GetModuleHandle_(0)
      Minor = GetVersion_()
      Major = (Minor & $FF)
      Minor = (Minor & $FF00) >> 8
  
      CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
        If (Major >= 6)
          IsMinWindowsVista = #True
          If (Major = 6 And Minor >= 1) Or (Major > 6)
            IsMinWindows7 = #True
            EndIf
          EndIf
        CompilerElse
        CompilerIf #PB_Compiler_Unicode
          If (Major >= 5)
            IsMinWindows2000 = #True
            If (Major > 5) Or (Minor >= 1)
              IsMinWindowsXP = #True
              If (Major >= 6)
                IsMinWindowsVista = #True
                If (Major = 6 And Minor >= 1) Or (Major > 6)
                  IsMinWindows7 = #True
                  EndIf
                EndIf
              EndIf
            EndIf
          CompilerElse
          If (Major > 4 Or (Major = 4 And Minor >= 10))
            IsMinWindows98 = #True
            If Minor >= 90
              IsMinWindowsME = #True
              EndIf
            EndIf
          CompilerEndIf
        xLong1 = GetDC_(0)
        If xLong1
          If GetDeviceCaps_(xLong1, 12) > 15
            IsHighColor = #True
            EndIf
          ReleaseDC_(0, xLong1)
          EndIf
        CompilerEndIf


CompilerIf #PB_Compiler_ExecutableFormat = #PB_Compiler_Executable And #PB_Compiler_Unicode
  CompilerIf Not #PB_Compiler_Processor = #PB_Processor_x64
      If IsMinWindowsXP
  CompilerEndIf
        xLong1 = LoadLibrary_(@"shell32.dll")
        InitCommonControls_()
        FreeLibrary_(xLong1)
  CompilerIf Not #PB_Compiler_Processor = #PB_Processor_x64
        EndIf
  CompilerEndIf
CompilerEndIf


      LibBuild_CondInitModule(AppBase)


      CompilerIf Not #PB_Compiler_Processor = #PB_Processor_x64
CompilerIf #PB_Compiler_Unicode
        If GetVersion_() & $80000000
;CompilerElse
;        If Not GetVersion_() & $80000000
;CompilerEndIf
;216 The image file %1 is valid, but is For a machine type other than the current machine.  ERROR_EXE_MACHINE_TYPE_MISMATCH 
          Minor = FormatMessage_(#FORMAT_MESSAGE_FROM_SYSTEM | #FORMAT_MESSAGE_IGNORE_INSERTS, 0, #ERROR_EXE_MACHINE_TYPE_MISMATCH, 0, *ptrTemp, #ptrTempSizeXAW, 0)
          If Minor
            xChar1 = PeekS(*ptrTemp, Minor)
            If Not Len(AppName)
              CompilerIf Defined(LibBuild_GetFileVersionField_Included, #PB_Constant)
                AppName = GetFileVersionField(#GetFileVersionField_ProductName)
                If Not Len(AppName)
                  AppName = "???"
                  EndIf
                CompilerElse
                AppName = "???"
                CompilerEndIf
              EndIf
            Else
            CompilerIf #PB_Compiler_Unicode
              xChar1 = "This is a unicode executable, which will not run on Windows 95/98/ME."
              ;CompilerElse
              ;xChar1 = "This executable is for Windows 95/98/ME, which will not run on this version of Windows."
              CompilerEndIf
            If Not Len(AppName)
              CompilerIf Defined(LibBuild_GetFileVersionField_Included, #PB_Constant)
                AppName = GetFileVersionField(#GetFileVersionField_ProductName)
                If Not Len(AppName)
                  AppName = "this product"
                  EndIf
                CompilerElse
                AppName = "this product"
                CompilerEndIf
              EndIf
            EndIf
          xChar1 = ReplaceString(xChar1, "%1", AppName)
          MessageBox_(0, @xChar1, 0, #MB_ICONEXCLAMATION)
          MemFree(*ptrTemp)
          *ptrTemp = 0
          ProcedureReturn 0
          EndIf
CompilerEndIf
        CompilerEndIf

      CompilerIf Defined(InitCommonControls_ProgressBar, #PB_Constant)
        #InitCommonControls_Do = #True
        CompilerElseIf Defined(InitCommonControls_TreeView, #PB_Constant)
        #InitCommonControls_Do = #True
        CompilerElseIf Defined(InitCommonControls_ListView, #PB_Constant)
        #InitCommonControls_Do = #True
        CompilerEndIf
      CompilerIf Defined(InitCommonControls_Do, #PB_Constant)
        Define.initcommoncontrolsex xType1
        CompilerIf Defined(InitCommonControls_ProgressBar, #PB_Constant)
          xLong1 | #InitCommonControls_ProgressBar
          CompilerEndIf
        CompilerIf Defined(InitCommonControls_TreeView, #PB_Constant)
          xLong1 | #InitCommonControls_TreeView
          CompilerEndIf
        CompilerIf Defined(InitCommonControls_ListView, #PB_Constant)
          xLong1 | #InitCommonControls_ListView
          CompilerEndIf
        xType1\dwSize = SizeOf(initcommoncontrolsex)
        xType1\dwICC = xLong1
        InitCommonControlsEx_(xType1)
        InitCommonControls_()
        CompilerEndIf
  
      CompilerIf Defined(CoInitialize, #PB_Constant)
        CoInitialize_(0)
        CompilerEndIf

      ;;; done already !!! LibBuild_CondInit(AppBase)
      CompilerIf Defined(WaitableTimer_Init, #PB_Constant)
        WaitableTimer_LibBuildPostInit()
        CompilerEndIf
      LibBuild_CondInitModule(RegisterDeviceNotification_Main)
      LibBuild_CondInitModule(DPIScaling)
      LibBuild_CondInitModule(ThemingAPI)
      LibBuild_CondInitModule(WinSock_Base)
      LibBuild_CondInitModule(WinSock_HTTPReceiverMT)
      ProcedureReturn #True
      EndIf
    EndProcedure
    
CompilerIf Defined(LibBuild_GetCursorPosPX_Included, #PB_Constant)
  CompilerIf Defined(LibBuild_SetProcessDPIAware_Included, #PB_Constant)
    GetCursorPosPX_DPIAware()
    CompilerElseIf Defined(LibBuild_SetProcessDpiAwareness_Included, #PB_Constant)
    GetCursorPosPX_DPIAware()
    CompilerElse
    GetCursorPosPX_DPIUnaware()
    CompilerEndIf
  CompilerEndIf

  Procedure LibBuild_Close()
    CompilerIf Defined(LibBuild_HtmlHelp_Included, #PB_Constant)
      If HtmlHelp_LastCall
        HtmlHelpClose()
        EndIf
      CompilerEndIf
    CompilerIf Defined(WinSock_HTTPReceiverMT_CreateSaveThreadFromURL, #PB_Constant)
      If WinsockHTTPReceiverMT_SaveWindowHwnd
        DestroyWindow_(WinsockHTTPReceiverMT_SaveWindowHwnd)
        EndIf
      UnregisterClass_(#WinsockHTTPReceiverMT_SaveWindowClass, hInstance)
      CompilerEndIf
    CompilerIf Defined(LibBuild_WinSock_Base_Included, #PB_Constant)
      WinSock_Close()
      CompilerEndIf
    If *ptrTemp
      MemFree(*ptrTemp)
      EndIf

    CompilerIf #PB_Compiler_Debugger And #LibBuild_DebugMemory
      If MemCounts
        Debug "Unfreed memory allocs: " + Str(MemCounts)
        EndIf
      CompilerEndIf
    EndProcedure
  EndMacro

Procedure DoEvents()
  Define.MSG xMsg
  While PeekMessage_(@xMsg, 0, 0, $3ff, #PM_REMOVE)
    TranslateMessage_(@xMsg)
    DispatchMessage_(@xMsg)
    Wend
  EndProcedure

; IDE Options = PureBasic 5.31 (Windows - x64)
; CursorPosition = 381
; FirstLine = 366
; Folding = ------
; Executable = CleanupEmptyDirs.exe
; CommandLine = "T:\GameMusicRipp\2011\Alien Breed 2 - Assault"
; CompileSourceDirectory